#!/bin/env perl -w
###################################################################
###                                                             ###
###      File: Makefile.PL                                      ###
###   Version:                                                  ###
###    Author: Stefan Evert                                     ###
###   Purpose: Perl Makefile for module.                        ###
###   Created: Fri Dec 11 17:18:31 1998                         ###
###  Modified: Fri Jan 21 12:55:57 2005 (evert)                 ###
###                                                             ###
###################################################################
#
# History:
#


use ExtUtils::MakeMaker;
use FileHandle;

# insert the directory in which CWB tools and CQP are installed if auto-config fails
# $CWB_Path = "/usr/local/bin";

#
#  do not edit anything below this point
#

# required CWB version for this module (3.0, but 2.2.b85 should work, too)
$rMajor = 2;
$rMinor = 2;
$rBeta = 85;

# try to find directory containing up-to-date CWB binaries
if (not defined $CWB_Path) {
  if ($ENV{PATH}) {
    @Locations = split /:/, $ENV{PATH};	# look for CQP binary in user's search path
  }
  else {			# try some standard locations
    @Locations = ("/usr/local/bin", "/usr/bin");
  }
  print "Searching for CWB binaries ...\n";
  foreach $dir (@Locations) {
    if (-x "$dir/cqp") { # use CQP to identify the correct directory
      if ((`$dir/cqp -v | grep Version:`) =~ /([0-9])\.([0-9])(\.b([0-9]+)\+?)?/) {
	$MajorVersion = $1;
	$MinorVersion = $2;
	$BetaVersion = $4;
	if ($MajorVersion > $rMajor or 
	    ($MajorVersion == $rMajor and ($MinorVersion > $rMinor or 
					   $MinorVersion == $rMinor and $BetaVersion >= $rBeta))) {
	  $CWB_Path = $dir;
	  last;
	}
      }
    }
  }
}

if ($CWB_Path and -d $CWB_Path) {
  print "CWB binaries are installed in $CWB_Path\n";
}
else {
  die "ERROR: Can't locate CWB binaries. Please edit Makefile.PL, setting\n".
    "       \$CWB_Path to the directory where CQP and the CWB tools are installed.\n";
}

# clean up in libdir and substitute the correct CWB bindir into CWB.pm
@backups = glob "lib/*~";			# just to make sure there are no emacs backups around
foreach $file (@backups) {
  print "Deleting $file ... \n";
  unlink $file;
}
print "In-place editing 'lib/CWB.pm' ... \n";
$fh = new FileHandle "lib/CWB.pm";		# read CWB.pm into list
die "Can't read 'lib/CWB.pm'. Aborted.\n"
  unless defined $fh;
@lines = <$fh>;
die "Read error.\n"
    unless @lines >= 20;			# rough check :o)
$fh->close;
$substitutions = 0;				# should replace exactly 1 line in list
foreach $line (@lines) {
  if ($line =~ /^\s*\$BinDir\s*=.*--auto-edit--/) {
    $line = "\$BinDir = \"$CWB_Path\"; \# --auto-edit-- CWB binaries directory\n";
    $substitutions++;
  }
}
if ($substitutions != 1) {
  die " ! substitution error (made $substitutions changes). Aborted.\n";
}
$fh = new FileHandle "> lib/CWB.pm";		# write modified lines back to CWB.pm
die "Write error.\n"
  unless defined $fh;
map { $fh->print($_) } @lines;
$fh->close
  or die "Write error. 'CWB.pm' may be damaged!\n";

# having made the substitution, we can now write the makefile
WriteMakefile( 
	      NAME => "CWB",
	      VERSION_FROM => "lib/CWB.pm",
	      EXE_FILES => ["bin/cwb-make", "bin/cwb-regedit"],
	     );
