#!/usr/local/bin/perl -w
###################################################################
###                                                             ###
###    Author: Stefan Evert                                     ###
###   Purpose: Test WebCqp::Persistent module                   ###
###   Created: Mon Apr 21 11:25:43 2003                         ###
###  Modified: Mon Apr 21 11:54:33 2003 (evert)                 ###
###                                                             ###
###################################################################
$| = 1;

use CWB;
use CQP;
use WebCqp::Persistent;

die "Usage:  test.pl <name> [<query>]\n"
  unless @ARGV >= 1;

$name = shift @ARGV;
$query = "@ARGV";

$cqp = new CQP;
$cache = new WebCqp::Persistent $cqp, "/tmp/persistent", 1, 1/60;

$cqp->set_error_handler(sub { map {print STDERR "MY HANDLER: $_\n"} @_; exit 1; });


print "Contents of cache index directory:\n";
system "ls -l /tmp/persistent/index";

$size = $cache->retrieve("DICKENS:$name");
if (defined $size) {
  print "DICKENS:$name is persistent (size = $size)\n";
  exit 0;
}
unless ($query) {
  print "DICKENS:$name has expired from the cache.\n";
  exit 0;
}

if ($query eq "X") {
  # simulate internal error: trying to store non-existent query result
  $cqp->exec("set DataDirectory ''");
  $cache->store("DICKENS:$name-XXX");
  die "Should not have reached this point.";
}

print "Running query ... ";
$cqp->exec("DICKENS");
$cqp->exec("$name = $query");
($size) = $cqp->exec("size DICKENS:$name");
print "$size matches\n";

print "Making DICKENS:$name persistent ... ";
$id = $cache->store("DICKENS:$name");
print "saved as $id\n";

print "Contents of cache index directory:\n";
system "ls -l /tmp/persistent/index";
print "Contents of cache data directory:\n";
system "ls -l /tmp/persistent/data";

